////////////////////////////////////////////////////////////////////////////////
//
//  CLOUD CREATIVE STUDIOS, INC.
//
//  Copyright (C) 2011-2012 Cloud Creative Studios, Inc.
//  All Rights Reserved.
//
//  http://cloud-creative-studios.com
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///@brief 雑多なライブラリ群
///
///ここに配置される関数は、プロジェクトに依存しない、
///汎用化された関数を配置するべきである。
///プロジェクトに依存するようなコードは配置してはいけない。
ccs.Aux = {};

(function() {
	
	//--------------------------------------------------------------------------
	ccs.Aux.App = {};
	
	//--------------------------------------------------------------------------
	///@brief イベントリスナーの削除
	///@param [in] event 削除するイベント
	///@param [in] win イベントを削除するトリガを発生させるウィンドウ
	///@param [in] handler イベントを削除するトリガとなるイベント
	///@param [in] callback 削除するイベントに割り当てられているコールバック関数
	///
	///イベント削除のタイミングをディレイさせる。
	///
	///@bug
	///iPhoneでは同じイベントに対して、複数のコールバックを指定することが可能だが、
	///Androidの場合は、コールバックは一つしか登録できないため、
	///既に登録済みの @p handler に対して、この関数を指定した場合は、
	///登録済みのハンドラが上書きされてしまう。
	ccs.Aux.App.removeEventListener = function(event, win, handler, callback) {
		win.addEventListener(handler, function() {
			Ti.App.removeEventListener(event, callback);
		});
	};

	//--------------------------------------------------------------------------
	///@brief Ti.App.addEventListenerをラップしてOneShot動作を可能にする
	///@param eventName イベント名
	///@param eventHandler イベント時に呼び出す関数
	ccs.Aux.App.addEventListener = function(eventName, eventHandler) {
		var proxyHandler = function(e) {
			ccs.DebugLog("ccs.Aux.App.proxyHandler");
			eventHandler(e);
			removeEventListener();
		};
		function removeEventListener() {
			ccs.DebugLog("ccs.Aux.App.removeEventListener : " + eventName);
			Ti.App.removeEventListener(eventName, proxyHandler);
		}
		Ti.App.addEventListener(eventName, proxyHandler);
	};
	
	//--------------------------------------------------------------------------
	ccs.Aux.UI = {};
	ccs.Aux.UI.iPhone = {};
	ccs.Aux.UI.Android = {};
	
	//--------------------------------------------------------------------------
	ccs.Aux.UI.iPhone.statusBarHeight = 20;
	ccs.Aux.UI.Android.statusBarHeightMedium = 25;
	ccs.Aux.UI.statusBarWidth = 320;
	ccs.Aux.UI.NavBarHeight = 44;
	
	//--------------------------------------------------------------------------
	///@brief 古いインクルードスタイルか返す
	///@retval true 古いスタイル
	///@retval false 新しいスタイル
	///
	///@note
	///Titanium SDKの1.7.0未満のiPhone環境では、
	///Ti.include()で相対パス指定が出来ない。
	ccs.Aux.isIncludedOldStyle = function() {
		return Ti.Platform.osname === "iphone" && Ti.version < "1.7.0";
	};
	
	//--------------------------------------------------------------------------
	ccs.Aux.Platform = {};
	
	//--------------------------------------------------------------------------
	///@brief iPhoneで動作しているか返す
	///@retval true iPhone上で動作している
	///@retval false iPhone上で動作していない
	ccs.Aux.Platform.isIPhone = function() {
		return Ti.Platform.osname === "iphone";
	};
	
	//--------------------------------------------------------------------------
	///@brief iOSバージョンをチェック
	///@param [in] version メジャーバージョン
	///@retval true 指定バージョン以降
	///@retval false 指定バージョン未満
	///
	///@p version に指定されたメジャーバージョン以降のバージョンか返す。
	var checkIOSMajorVersion = function(version) {
		if (ccs.Aux.Platform.isIPhone()) {
			var version = Ti.Platform.version.split(".");
			var major = parseInt(version[0]);
			
			if (major >= version) {
				return true;
			}
		}
		
		return false;
	};
	
	//--------------------------------------------------------------------------
	ccs.Aux.monitorTimer = null;
	ccs.Aux.monitorTimerInterval = null;
	var monitorHandler = function() {
		ccs.DebugLog("Available Memory -> " + Ti.Platform.availableMemory);
		ccs.DebugLog("Battery Level -> " + Ti.Platform.batteryLevel);
		setTimeout(monitorHandler, ccs.Aux.monitorTimerInterval);
	};
	
	//--------------------------------------------------------------------------
	ccs.Aux.startDebugMonitor = function(interval) {
		Ti.Platform.batteryMonitoring = true;
		ccs.Aux.monitorTimerInterval = interval;
		ccs.Aux.monitorTimer = setTimeout(monitorHandler, ccs.Aux.monitorTimerInterval);
	};
	
	//--------------------------------------------------------------------------
	ccs.Aux.stopDebugMonitor = function() {
		clearTimeout(ccs.Aux.monitorTimer);
	};
	
	//--------------------------------------------------------------------------
	///@brief iPhone4以降か返す
	///@retval true iPhone4以降
	///@retval false iPhone以前又はiPhoneではない 
	ccs.Aux.Platform.isIOS4Plus = function() {
		return checkIOSMajorVersion(4);
	};
	
	//--------------------------------------------------------------------------
	///@brief iOS5か返す
	///@retval true iOS5以降
	///@retval false iOS5以前又はiOSではない 
	ccs.Aux.Platform.isIOS5 = function() {
		return checkIOSMajorVersion(5);
	};
	
	//--------------------------------------------------------------------------
	///@brief iPadで動作しているか返す
	///@retval true iPad上で動作している
	///@retval false iPad上で動作していない
	ccs.Aux.Platform.isIPad = function() {
		return Ti.Platform.osname === "ipad";
	};
	
	//--------------------------------------------------------------------------
	///@brief Androidで動作しているか返す
	///@retval true Android上で動作している
	///@retval false Android上で動作していない
	ccs.Aux.Platform.isAndroid = function() {
		return Ti.Platform.osname === "android";
	};
	
	//--------------------------------------------------------------------------
	///@brief シミュレータで動作しているか返す
	///@retval true シミュレータ上で動作している
	///@retval false シミュレータ上で動作していない
	ccs.Aux.Platform.isSimulator = function() {
		if (ccs.Aux.Platform.isIPhone()) {
			return Ti.Platform.model === "Simulator";
		} else {
			//Androidエミュレーターで、SDKにGoogleAPIを使用するとgoogle_sdkが返される。
			//そうでないときはsdk
			return (Ti.Platform.model === "google_sdk" || Ti.Platform.model === "sdk");
		}
	};
	
}());
